

set -ex



pip check
nbgrader -h
jupyter lab --version
jupyter labextension list
jupyter server extension list
jupyter labextension list 1>labextensions 2>&1
jupyter server extension list 1>server_extensions 2>&1
grep -iE "nbgrader.server_extensions.assignment_list.*OK" server_extensions
grep -iE "nbgrader.server_extensions.assignment_list.*enabled" server_extensions
grep -iE "nbgrader.server_extensions.course_list.*OK" server_extensions
grep -iE "nbgrader.server_extensions.course_list.*enabled" server_extensions
grep -iE "nbgrader.server_extensions.formgrader.*OK" server_extensions
grep -iE "nbgrader.server_extensions.formgrader.*enabled" server_extensions
grep -iE "nbgrader.server_extensions.validate_assignment.*OK" server_extensions
grep -iE "nbgrader.server_extensions.validate_assignment.*enabled" server_extensions
grep -iE "nbgrader.*OK.*nbgrader" labextensions
exit 0
